
<<setup_workspace.m;
<<setup_params.m;
<<setup_shocks.m;
<<setup_grids.m;

(* Construct the expected value function *)

OmegaTm1Raw[s_] := (1/NumOfyShockPoints) Sum[u[R*s + yShockVals[[i]]],{i,Length[yShockVals]}];


(* For each value of xGrid, calculate the maximum utility and optimal c *)

MaxResultsTm1 =
  Table[
    tmpResults = 
      FindMinimum[                           (* There's no built-in FindMaximum command *)
        -(u[c] + beta OmegaTm1Raw[xGrid[[i]]-c])  (* So we find the minimum of -V            *)
        ,{c,{.01,xGrid[[i]]-.01}}];
(* The FindMinimum command returns {minimum,{control->optimum}}
   The code below extracts the two elements
*)
     {tmpResults[[1]],(c /. tmpResults[[2]])}
   ,{i,Length[xGrid]}];
    

MaxValuesTm1 = -Transpose[MaxResultsTm1][[1]]; (* Extract the maxima      *)
cValuesTm1   =  Transpose[MaxResultsTm1][[2]]; (* Extract the optimal c's *)

(* Construct the numerical interpolating functions *)
VTm1 = Interpolation[Transpose[{xGrid,MaxValuesTm1}],InterpolationOrder->1];
cTm1 = Interpolation[Transpose[{xGrid,cValuesTm1}],InterpolationOrder->1];

cTm1CEQ[x_] := .5 + x/2;  (* Optimal policy if there is no uncertainty is consume half *)

PlotVTm1Simple = 
    Plot[VTm1[x],{x,xMin,xMax}
         ,PlotLabel->"VTm1"
         ,ImageSize->{72 6.,72 6./GoldenRatio}
         ];
Display[ParentDirectoryString<>"PlotVTm1Simple.EPS",PlotVTm1Simple,"EPS"];

PlotcTm1Simple = 
    Plot[{cTm1[x],cTm1CEQ[x]},{x,xMin,xMax}
         ,PlotStyle->{Dashing[{}],GrayLevel[.5]}
         ,PlotLabel->"cTm1[XTm1] (solid line) vs. cTm1CEQ[XTm1] (gray)"
         ,ImageSize->{72 6.,72 6./GoldenRatio}
         ];
Display[ParentDirectoryString<>"PlotcTm1Simple.EPS",PlotcTm1Simple,"EPS"];