<<setup_workspace.m;
<<setup_params.m;
<<setup_grids.m;
<<setup_shocks.m;

(* Construct the expected value function *)

OmegaTm1Raw[s_] := (1/NumOfyShockPoints) Sum[u[R*s + yShockVals[[i]]],{i,Length[yShockVals]}];

(* Calculate the value of OmegaTm1Raw for all the points in sGrid *)

OmegaTm1Data = Table[OmegaTm1Raw[sGrid[[i]]],{i,Length[sGrid]}];

(* Construct a numerical interpolating function connecting the points in (sGrid,OmegaTm1Data) *)
OmegaTm1     = Interpolation[Transpose[{sGrid,OmegaTm1Data}],InterpolationOrder->1];

PlotOTm1RawVSInt = Plot[
  {OmegaTm1Raw[s],OmegaTm1[s]}
  ,{s,xMin,xMax}
  ,PlotStyle->{Dashing[{}],Dashing[{.01}]}
  ,PlotLabel->"\[CapitalOmega]Tm1Raw[s] versus \[CapitalOmega]Tm1[s]"
  ,ImageSize->{72 6.,72 6./GoldenRatio}
];
Display[ParentDirectoryString<>"PlotOTm1RawVSInt.EPS",PlotOTm1RawVSInt,"EPS"];

(* For each value of xGrid, calculate the optimal c and the utility it yields *)

MaxResultsTm1 =
  Table[
    tmpResults = 
      FindMinimum[                           (* There's no built-in FindMaximum command *)
        -(u[c] + beta OmegaTm1[xGrid[[i]]-c]) (* So we find the minimum of -V            *)
        ,{c,{.01,xGrid[[i]]-.01}}];
(* The FindMinimum command returns {minimum,{control->optimum}}
   The code below extracts the two elements
*)
     {tmpResults[[1]],(c /. tmpResults[[2]])}
   ,{i,Length[xGrid]}];
    

MaxValuesTm1 = -Transpose[MaxResultsTm1][[1]]; (* Extract the maxima      *)
cValuesTm1   =  Transpose[MaxResultsTm1][[2]]; (* Extract the optimal c's *)

(* Construct numerical interpolating functions for VTm1 and cTm1 *)
VTm1 = Interpolation[Transpose[{xGrid,MaxValuesTm1}],InterpolationOrder->1];
cTm1 = Interpolation[Transpose[{xGrid,cValuesTm1}],InterpolationOrder->1];

PlotVTm1IntExp = 
  Plot[VTm1[x],{x,xMin,xMax}
    ,PlotStyle->Dashing[{.01}]
    ,DisplayFunction->Identity];
PlotcTm1IntExp = 
  Plot[cTm1[x],{x,xMin,xMax}
  ,PlotStyle->{Dashing[{.01}]}
  ,DisplayFunction->Identity];

PlotCompareVTm1AB = 
  Show[PlotVTm1Simple,PlotVTm1IntExp
     ,DisplayFunction->$DisplayFunction
     ,ImageSize->{72 6.,72 6./GoldenRatio}
     ,PlotLabel->"VTm1 Calculated Using \[CapitalOmega]Tm1Raw and \[CapitalOmega]Tm1"
     ];
Display[ParentDirectoryString<>"PlotCompareVTm1AB.EPS",PlotCompareVTm1AB,"EPS"];

PlotComparecTm1AB = 
  Show[PlotcTm1IntExp,PlotcTm1Simple
     ,DisplayFunction->$DisplayFunction
     ,ImageSize->{72 6.,72 6./GoldenRatio}
     ,PlotLabel->"cTm1 Calculated Using \[CapitalOmega]Tm1Raw and \[CapitalOmega]Tm1"
     ];
Display[ParentDirectoryString<>"PlotComparecTm1AB.EPS",PlotComparecTm1AB,"EPS"];

xEq3Plot = Plot[
	{(OmegaTm1Raw[3-c+.001]-OmegaTm1Raw[3-c-.001])/.002
	,(OmegaTm1[3-c+.001]-OmegaTm1[3-c-.001])/.002}
,{c,.1,3}
,PlotRange->{{0,3},{0,0.8}}
,PlotStyle->{Dashing[{}],Dashing[{}],Dashing[{}]}
,ImageSize->{72 6.,72 6./GoldenRatio}
,DisplayFunction->Identity
];

xEq4Plot = Plot[
	{uprime[c]
    ,(OmegaTm1Raw[4-c+.001]-OmegaTm1Raw[4-c-.001])/.002
	,(OmegaTm1[4-c+.001]-OmegaTm1[4-c-.001])/.002}
,{c,.1,4}
,PlotRange->{{0,4},{0,0.8}}
,PlotStyle->{Thickness[.006],Dashing[{.01}],Dashing[{.01}]}
,ImageSize->{72 6.,72 6./GoldenRatio}
,DisplayFunction->Identity
];

PlotuPrimeVSOPrime = 
Show[xEq3Plot,xEq4Plot,PlotRange->{{0,4},{0,0.8}}
,ImageSize->{72 6.,72 6./GoldenRatio}
,PlotLabel->"u'[c] vs {\[CapitalOmega]'Tm1Raw[3-c],\[CapitalOmega]'Tm1[3-c]},{\[CapitalOmega]'Tm1Raw[4-c],\[CapitalOmega]'Tm1[4-c]}"
,DisplayFunction->$DisplayFunction
,AxesLabel -> {"cTm1",""}]

Display[ParentDirectoryString<>"PlotuPrimeVSOPrime.EPS",PlotuPrimeVSOPrime,"EPS"];
