<<setup_workspace.m;
<<setup_params.m;
<<setup_grids.m;
<<setup_shocks.m;

(* Construct the expected value functions *)

OmegaTm1Raw[s_]      :=  (1/NumOfyShockPoints) Sum[     u[R*s + yShockVals[[i]]],{i,Length[yShockVals]}];
OmegaPrimeTm1Raw[s_] := R(1/NumOfyShockPoints) Sum[uprime[R*s + yShockVals[[i]]],{i,Length[yShockVals]}];

(* Construct an interpolation for the expected value of saving *)

sGrid = xGrid;
OmegaPrimeTm1Data = Table[OmegaPrimeTm1Raw[sGrid[[i]]],{i,Length[sGrid]}];
OmegaPrimeTm1 = Interpolation[Transpose[{sGrid,OmegaPrimeTm1Data}],InterpolationOrder->1];

PlotOPrimeRawVSFOC =
  Plot[
  {OmegaPrimeTm1Raw[s],OmegaPrimeTm1[s]}
  ,{s,xMin,xMax}
  ,PlotStyle->{Dashing[{}],Dashing[{.01}]}
  ,PlotLabel->"\[CapitalOmega]'Tm1Raw[s] versus \[CapitalOmega]'Tm1[s]"
  ,ImageSize->{72 6.,72 6./GoldenRatio}
];
Display[ParentDirectoryString<>"PlotOPrimeRawVSFOC.EPS",PlotOPrimeRawVSFOC,"EPS"];

(* For each value of xGrid, calculate the maximum utility and optimal c *)

FOCcValuesTm1 =
  Table[
    tmpResults = 
      FindRoot[
        uprime[c] == beta OmegaPrimeTm1[xGrid[[i]]-c]
        ,{c,{.01,xMax}}];
(* The FindRoot command returns {control->optimum}
   The code below extracts the two elements
*)
     (c /. tmpResults)
   ,{i,Length[xGrid]}];
    

cTm1FOC = Interpolation[Transpose[{xGrid,FOCcValuesTm1}],InterpolationOrder->1];

PlotcTm1IntExpFOC = 
    Plot[{cTm1FOC[x]},{x,xMin,xMax}
    ,PlotStyle->Dashing[{.02}]
    ,DisplayFunction->Identity
    ];

PlotcTm1ABC =
  Show[PlotcTm1IntExp,PlotcTm1Simple,PlotcTm1IntExpFOC
    ,DisplayFunction->$DisplayFunction
    ,ImageSize->{72 6.,72 6./GoldenRatio}
    ,PlotLabel->"Comparison of Three Methods for Finding cTm1[x]"
    ];
Display[ParentDirectoryString<>"PlotcTm1ABC.EPS",PlotcTm1ABC,"EPS"];
