<<setup_workspace.m;
<<setup_params.m;
<<setup_grids.m;
<<setup_shocks.m;

(* Construct the expected value function and expected marginal value  *)

OmegaTm1Raw[s_]      :=  (1/NumOfyShockPoints) Sum[     u[R*s + yShockVals[[i]]],{i,Length[yShockVals]}];
OmegaPrimeTm1Raw[s_] := R(1/NumOfyShockPoints) Sum[uprime[R*s + yShockVals[[i]]],{i,Length[yShockVals]}];

(* Construct an interpolation for the expected value of saving *)

OmegaInvPrimeTm1Data = Table[(OmegaPrimeTm1Raw[sGrid[[i]]])^(-1/rho),{i,Length[sGrid]}];
OmegaInvPrimeTm1     = Interpolation[Transpose[{sGrid,OmegaInvPrimeTm1Data}],InterpolationOrder->1];

PlotOPInvVSOInvP =
  Plot[
  {(OmegaPrimeTm1Raw[s])^(-1/rho),OmegaInvPrimeTm1[s]}
  ,{s,xMin,xMax}
  ,PlotStyle->{Dashing[{}],Dashing[{.01}]}
  ,PlotLabel->"\[CapitalOmega]'Tm1Raw[s]^(-1/\[Rho]) versus \[Mho]'Tm1[s]"
  ,ImageSize->{72 6.,72 6./GoldenRatio}
];
Display[ParentDirectoryString<>"PlotOPInvVSOInvP.EPS",PlotOPInvVSOInvP,"EPS"];

PlotOPVSOInvPInv = 
  Plot[
  {OmegaPrimeTm1Raw[s],OmegaInvPrimeTm1[s]^-rho}
  ,{s,xMin,xMax}
  ,PlotStyle->{Dashing[{}],Dashing[{.01}]}
  ,PlotLabel->"\[CapitalOmega]'Tm1Raw[s] versus \[Mho]'Tm1[s]^-\[Rho]"
  ,ImageSize->{72 6.,72 6./GoldenRatio}
];
Display[ParentDirectoryString<>"PlotOPVSOInvPInv.EPS",PlotOPVSOInvPInv,"EPS"];


(* For each value of xGrid, calculate the maximum utility and optimal c *)

FOCInvcValuesTm1 =
  Table[
    tmpResults = 
      FindRoot[
        uprime[c] == beta (OmegaInvPrimeTm1[xGrid[[i]]-c])^-rho
        ,{c,{.001,xGrid[[i]]+.01}}];
(* The FindRoot command returns {control->optimum}
   The code below extracts the two elements
*)
     (c /. tmpResults)
   ,{i,Length[xGrid]}];
    

cTm1FOCInv = Interpolation[Transpose[{xGrid,FOCInvcValuesTm1}],InterpolationOrder->1];

PlotcTm1IntExpFOCInv = 
    Plot[{cTm1FOCInv[x]},{x,xMin,xMax}
    ,PlotStyle->Dashing[{.02}]
    ,DisplayFunction->Identity
    ];

PlotComparecTm1AD = 
  Show[PlotcTm1Simple,PlotcTm1IntExpFOCInv
    ,DisplayFunction->$DisplayFunction
    ,ImageSize->{72 6.,72 6./GoldenRatio}
    ,PlotLabel->"Comparison of cTm1[x] with cTm1FOCInv[x]"
    ];
Display[ParentDirectoryString<>"PlotComparecTm1AD.EPS",PlotComparecTm1AD,"EPS"];
