(*

This file makes a figure illustrating the discrete approximation to the
lognormal distribution used in the class notes on solving dynamic
optimization problems

*)

<<setup_workspace.m

{yShockVals,yShockProbs} = DiscreteApproxToMeanOneLogNormal[.1,7];

hMin = yShockVals[[1]]-.1;
hMax = yShockVals[[-1]]+.1;

CDFPlot = Plot[
CDF[LogNormalDistribution[LevelAdjustingParameter,.1],x]
,{x,hMin,hMax}
,PlotRange->{{hMin,hMax},{0,1}}
,AxesOrigin->{hMin,0}
];

HorizLines = 
Table[
  Plot[
  CDF[LogNormalDistribution[LevelAdjustingParameter,.1],ListOfEdgePoints[[LoopOverEdges]]]
  ,{h,hMin,hMax}
  ,PlotStyle->Dashing[{.01}]
  ]
,{LoopOverEdges,2,Length[ListOfEdgePoints]}]

MassPoints = 
Table[Point[
{yShockVals[[i]],
  If[i>1,
	(CumulativeSums[yShockProbs][[i]]-CumulativeSums[yShockProbs][[i-1]])/2
	+CumulativeSums[yShockProbs][[i-1]],
    CumulativeSums[yShockProbs][[1]]/2]}]
,{i,Length[yShockVals]}];

MassPointPlot = Graphics[{PointSize[.02],MassPoints}];
 
Show[HorizLines,CDFPlot,MassPointPlot
,AxesOrigin->{hMin,0}
,AxesLabel->{"\[Epsilon]","CDF"}
,Label->"Discrete Approximation to Lognormal"];
