<<setup_workspace.m;
<<setup_params.m;
<<setup_grids.m;
<<setup_shocks.m;

<<multiperiod_vars.m;
<<multiperiod_functions.m;

beta = .95;             (* Make them slightly impatient *)
LifePosn = LifeLength;  (* Set LifePosn indicator value to the last period of life *)

xGridBaseline = xGrid;  (* Set up baseline value for state grid xGrid *)

While[LifePosn>0,

Print[LifePosn];

OmegaInvtArgumentList           = AddOutcome[OmegaInvtArgArray,Table[LifePosn,{Length[OmegaInvtArgArray]}]];
OmegaInvtRawResults             = Map[Apply[OmegaInvtRaw,#] &,OmegaInvtArgumentList];
OmegaInvtInterpData[[LifePosn]] = AddOutcome[OmegaInvtArgArray,OmegaInvtRawResults];
OmegaInvtInterpFunc[[LifePosn]] = Interpolation[OmegaInvtInterpData[[LifePosn]],InterpolationOrder->1];

OmegaInvPrimetArgumentList           = AddOutcome[OmegaInvPrimetArgArray,Table[LifePosn,{Length[OmegaInvPrimetArgArray]}]];
OmegaInvPrimetRawResults             = Map[Apply[OmegaInvPrimetRaw,#] &,OmegaInvPrimetArgumentList];
OmegaInvPrimetInterpData[[LifePosn]] = AddOutcome[OmegaInvPrimetArgArray,OmegaInvPrimetRawResults];
OmegaInvPrimetInterpFunc[[LifePosn]] = Interpolation[OmegaInvPrimetInterpData[[LifePosn]],InterpolationOrder->1];

(* Now find the point at which the liquidity constraint begins to bind *)
If[LifePosn < LifeLength,
  BindingPoint = (beta^(1/-rho) )OmegaInvPrimetInterpFunc[[LifePosn]][0];
  xGrid = Union[{BindingPoint},xGridBaseline];  (* Add the point to the gridpoints *)
  MakeArgArrays[xFunctions];                    (* Reconstruct the ArgArrays *)
];

ctArgumentList           = AddOutcome[ctArgArray,Table[LifePosn,{Length[ctArgArray]}]];
ctResults                = Map[Apply[ctRaw,#] &,ctArgumentList];
ctInterpData[[LifePosn]] = AddOutcome[ctArgArray,ctResults];
ctInterpFunc[[LifePosn]] = Interpolation[ctInterpData[[LifePosn]],InterpolationOrder->1];

LifePosn--;
] (* End While *)

LifePosn++;

TableOfcFuncs = 
  Table[
  Plot[
    ct[x,LP]
    ,{x,xMin,xMax}
    ,ImageSize->{72 6.,72 6./GoldenRatio}
    ,DisplayFunction->Identity
    ]
  ,{LP,LifeLength,1,-1}];

PlotCFuncsConverge = Show[
     TableOfcFuncs[[20]]
     ,TableOfcFuncs[[15]]
     ,TableOfcFuncs[[10]]
     ,TableOfcFuncs[[5]]
     ,TableOfcFuncs[[1]]
,DisplayFunction->$DisplayFunction
,PlotLabel->"ct[x,LifePosn] for LifePosn = 20,15,10,5,1"
];

Display[ParentDirectoryString<>"PlotCFuncsConverge.EPS",PlotCFuncsConverge,"EPS"];
