(*
This file contains various background routines that are needed for 
solving the dynamic stochastic optimization model
*)

<<DiscreteMath`Permutations`
<<Statistics`DataManipulation`
<<Statistics`DescriptiveStatistics`
<<Statistics`ContinuousDistributions`

If[$MachineType == "Macintosh",ParentDirectoryString = "::",ParentDirectoryString = ".."];

ClearAll[DiscreteApproxToMeanOneLogNormal];
DiscreteApproxToMeanOneLogNormal[StdDev_,NumOfPoints_] := Block[{},
LevelAdjustingParameter = -(1/2) (StdDev)^2;  (* This parameter takes on the value necessary to make the mean in levels = 1 *)
ListOfEdgePoints = Table[Quantile[LogNormalDistribution[LevelAdjustingParameter,StdDev],(i/NumOfPoints)],{i,NumOfPoints-1}];
ListOfEdgePoints = Flatten[{{0},ListOfEdgePoints,{Infinity}}];
ProbOfMeanPoints = Table[CDF[LogNormalDistribution[LevelAdjustingParameter,StdDev],ListOfEdgePoints[[i]]]
                        -CDF[LogNormalDistribution[LevelAdjustingParameter,StdDev],ListOfEdgePoints[[i-1]]]
                        ,{i,2,Length[ListOfEdgePoints]}];
ListOfMeanPoints = Table[NIntegrate[z PDF[LogNormalDistribution[LevelAdjustingParameter,StdDev],z],{z,ListOfEdgePoints[[i-1]],ListOfEdgePoints[[i]]}],{i,2,Length[ListOfEdgePoints]}] / ProbOfMeanPoints;
Return[{ListOfMeanPoints,ProbOfMeanPoints}]
]

ClearAll[MakeNewVarByAppendingStringToName];
MakeNewVarByAppendingStringToName[ListOfVarNames_,StringToAppend_] := Block[{},
 TableOfCommands = 
      Table[
        ListOfVarNames[[LoopOverVarNames]]<>StringToAppend<>" = Table[{},{i,"<>ToString[LifeLength]<>"}];"
        ,{LoopOverVarNames,Length[ListOfVarNames]}]; (* End Table *)
 Map[ToExpression,TableOfCommands];
]

ClearAll[MakeListFunctions];
MakeListFunctions[FunctionNameAndArgList_] :=    (* Create a function which maps f[{a,b,c}] to f[a,b,c] *)
Block[{},
Do[
  FunctionName = FunctionNameAndArgList[[LoopOverFunctions,NamePosInFunctionList   ]];
  FunctionArgs = FunctionNameAndArgList[[LoopOverFunctions,ArgListPosInFunctionList]];
  NumOfArgs = Length[FunctionArgs];
  CommandString = 
    "ClearAll["<>FunctionName<>"];"<>
    FunctionName<>"[ListOfArguments_] := "<>FunctionName<>"["<>
        Table["ListOfArguments[["<>ToString[LoopOverArgs]<>"]]"<>If[LoopOverArgs<NumOfArgs+1,",",""],{LoopOverArgs,NumOfArgs+1}]
    <>"];";
(*  Print[CommandString];*)
  ToExpression[CommandString];
,{LoopOverFunctions,Length[FunctionNameAndArgList]}
]
]

ClearAll[MakeFunctionsFromInterp];
MakeFunctionsFromInterp[FunctionNameAndArgList_] :=    (* Create a function which maps f[{a,b,c}] to f[a,b,c] *)
Block[{},
Do[
  FunctionName = FunctionNameAndArgList[[LoopOverFunctions,NamePosInFunctionList   ]];
  FunctionArgs = FunctionNameAndArgList[[LoopOverFunctions,ArgListPosInFunctionList]];
  NumOfArgs = Length[FunctionArgs];
  ListOfArgsAsFunctionArgs = StringJoin[Table[FunctionArgs[[i]]<>"_"<>If[i<NumOfArgs,",",""],{i,NumOfArgs}]];
  ListOfArgsAsVariables    = StringJoin[Table[FunctionArgs[[i]]<>If[i<NumOfArgs,",",""],{i,NumOfArgs}]];
  ToExpression[
    FunctionName<>"["<>ListOfArgsAsFunctionArgs<>",LifePosn_] := "<>FunctionName<>"InterpFunc[[Max[Min[EarliestPeriodSolved,EarliestEtVtp1Solved],LifePosn]]]["<>ListOfArgsAsVariables<>"]"
    ];
,{LoopOverFunctions,Length[FunctionNameAndArgList]}
]
]

MakeInterpGivenFunctionName[FunctionName_,InterpOrder_] := Block[{},
  ArgArray = FunctionName<>"ArgArray";
  Augmented = ArgArray<>"Augmented";
  ToExpression[
    Augmented<>" = AddOutcome["<>ArgArray<>",Table[LifePosn,{Length["<>ArgArray<>"]}]]"
    ];
    ToExpression[
    FunctionName<>"InterpData[[LifePosn]] = AddOutcome["<>ArgArray<>",Map[Apply["<>FunctionName<>"Raw,#] &,"<>Augmented<>"]]"
    ];
  ToExpression[
    FunctionName<>"InterpFunc[[LifePosn]] = Interpolation["<>FunctionName<>"InterpData[[LifePosn]],InterpolationOrder->"<>ToString[InterpOrder]<>"]"
    ];
]

MakeNormedInterpGivenFunctionName[FunctionName_,InterpOrder_,MultiplyBy_,NormFactor_] := Block[{},
  ArgArray = FunctionName<>"ArgArray";
  Augmented = ArgArray<>"Augmented";
  ToExpression[
    Augmented<>" = AddOutcome["<>ArgArray<>",Table[LifePosn,{Length["<>ArgArray<>"]}]]"
    ];
  ToExpression[
    FunctionName<>"InterpData[[LifePosn]] = AddOutcome["<>ArgArray<>",("<>ToString[MultiplyBy]<>"*Map[Apply["<>FunctionName<>"Raw,#] &,"<>Augmented<>"])^("<>ToString[NormFactor]<>")]"
    ];
  ToExpression[
    FunctionName<>"InterpFunc[[LifePosn]] = Interpolation["<>FunctionName<>"InterpData[[LifePosn]],InterpolationOrder->"<>ToString[InterpOrder]<>"]"
    ];
]

(* Constructs and executes a string of the form VtArgArray=Flatten[Outer[List,sGrid,hGrid,AggStateGrid,EmpStateGrid],3] *)
MakeArgArrays[FunctionList_] := Block[{},
Do[
  CurrentFunctionArgArray = FunctionList[[LoopOverFunctions,NamePosInFunctionList]]<>"ArgArray";
  ListOfArguments =    (* Makes a string of the form ",sGrid,hGrid,AggStateGrid,EmpStateGrid" *)
      Table[
          StringJoin[
            ","
            ,FunctionList[[LoopOverFunctions,ArgListPosInFunctionList,LoopOverArguments]]
            ,"Grid"]
        ,{LoopOverArguments,Length[FunctionList[[LoopOverFunctions,ArgListPosInFunctionList]]]}
          ];
  ToExpression[
      CurrentFunctionArgArray<>
      "=Flatten[Outer[List"<>StringJoin[ListOfArguments]<>"],"<>ToString[Length[FunctionList[[LoopOverFunctions,ArgListPosInFunctionList]]]-1]<>"]"
              ];
,{LoopOverFunctions,Length[FunctionList]}
]
];


(* % Adds a value corresponding to each set of values in the state grid *)
ClearAll[AddOutcome]; 
AddOutcome[StateGrid_,ValuesToAdd_] := Transpose[Join[Transpose[StateGrid],{ValuesToAdd}]];

LargestFirstElement[ListOfLists_] := Block[{},
  Largest = ListOfLists[[1]];
  Do[
  If[ListOfLists[[i,1]]>Largest[[1]],Largest=ListOfLists[[i]]];
  ,{i,2,Length[ListOfLists]}];
  Return[Largest]
];

SmallestFirstElement[ListOfLists_] := Block[{},
  Smallest = ListOfLists[[1]];
  Do[
  If[ListOfLists[[i,1]]<Smallest[[1]],Smallest=ListOfLists[[i]]];
  ,{i,2,Length[ListOfLists]}];
  Return[Smallest]
];
